﻿using Microsoft.AspNetCore.Builder;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using System;
using System.Collections.Generic;
using System.Reflection;

namespace Falcon.SugarApi.ApiDefinistions
{
    /// <summary>
    /// 模块注册
    /// </summary>
    public static class ModuleRegister
    {
        /// <summary>
        /// 模块服务注册
        /// </summary>
        /// <param name="services">服务集合</param>
        /// <param name="moduleName">模块名称</param>
        /// <param name="configuration">配置</param>
        /// <returns>服务集合</returns>
        /// <exception cref="Exception"></exception>
        public static IServiceCollection ModuleServiceRegistration(IServiceCollection services,string moduleName,IConfiguration configuration) {
            foreach(var mr in GetModuleRegister(moduleName)) {
                mr.ServiceRegister(services,configuration);
            }
            return services;
        }

        /// <summary>
        /// 应用注册
        /// </summary>
        /// <param name="application">应用创建器</param>
        /// <param name="moduleName">模块名称</param>
        /// <param name="configuration">配置</param>
        /// <returns>应用创建器</returns>
        public static IApplicationBuilder ApplicationBuilder(IApplicationBuilder application,string moduleName,IConfiguration configuration) {
            foreach(var mr in GetModuleRegister(moduleName)) {
                mr.ApplicationRegister(application,configuration);
            }
            return application;
        }

        private static IEnumerable<IModuleRegister> GetModuleRegister(string moduleName) {
            Assembly ass;
            try {
                ass = Assembly.Load(moduleName);
            }
            catch(Exception ex) {
                throw new Exception($"获取程序集{moduleName}错误！",ex);
            }
            var types = ass.GetTypes();
            foreach(var type in types) {
                if(type.GetInterface(typeof(IModuleRegister).FullName) != null) {
                    var obj = type.Assembly.CreateInstance(type.FullName);
                    if(obj is IModuleRegister mr) {
                        yield return mr;
                    }
                }
            }
        }
    }
}
